//
//  KochCurveViewController.m
//  KochCurve
//
//  Created by Clemens Wagner on 05.06.13.
//  Copyright (c) 2013 Clemens Wagner. All rights reserved.
//

#import "KochCurveViewController.h"
#import "KochView.h"
#import "Demo.h"
#import <QuartzCore/QuartzCore.h>

const CGFloat kPointRadius = 360;

@interface KochCurveViewController()

@property (weak, nonatomic) IBOutlet KochView *kochView;
@property (weak, nonatomic) IBOutlet UISlider *depthSlider;
@property (weak, nonatomic) IBOutlet UILabel *depthLabel;

@property (weak, nonatomic) IBOutlet UISlider *sizeSlider;
@property (weak, nonatomic) IBOutlet UILabel *sizeLabel;

- (IBAction)updateDepth:(UISlider *)inSender;
- (IBAction)updateDepthLabel:(UISlider *)inSender;
- (IBAction)updateSize:(UISlider *)inSender;
- (IBAction)updateSizeLabel:(UISlider *)inSender;

@end

@implementation KochCurveViewController

- (CALayer *)pointLayer {
    CALayer *theLayer = self.kochView.layer;
    
    return theLayer.sublayers[0];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    CGFloat theRadius = CGRectGetWidth(self.view.bounds) / 2.0;
    CALayer *theLayer = [CALayer layer];

    theLayer.backgroundColor = [[UIColor redColor] CGColor];
    theLayer.bounds = CGRectMake(0.0, 0.0, theRadius, theRadius);
    theLayer.cornerRadius = theRadius / 2.0;
    theLayer.borderColor = [[UIColor lightGrayColor] CGColor];
    theLayer.borderWidth = 1.0;
    theLayer.position = self.kochView.center;
    [self.kochView.layer addSublayer:theLayer];
    [self.kochView.layer setNeedsDisplay];
}

- (void)viewWillAppear:(BOOL)inAnimated {
    [super viewWillAppear:inAnimated];
    self.kochView.depth = self.depthSlider.value;
    [self updateDepthLabel:self.depthSlider];
    [self updateSize:self.sizeSlider];
}

- (IBAction)updateDepthLabel:(UISlider *)inSender {
    self.depthLabel.text = [NSString stringWithFormat:@"%.1f", inSender.value];
}

- (IBAction)updateDepth:(UISlider *)inSender {
    KochView *theView = self.kochView;
    CGFloat theDepth = roundf(inSender.value * 10.0) / 10.0;

    //
    // ======== START DEMO BEREICH ========
    //

#if DEMO == 3
    CAMediaTimingFunction *theFunction = [CAMediaTimingFunction functionWithControlPoints:0:2:1:-1];

    [CATransaction begin];
    [CATransaction setAnimationDuration:5.0];
    [CATransaction setAnimationTimingFunction:theFunction];
    theView.depth = theDepth;
    [CATransaction commit];
#elif DEMO == 2
    CABasicAnimation *theAnimation = [CABasicAnimation animation];
    CAMediaTimingFunction *theFunction = [CAMediaTimingFunction functionWithControlPoints:0:2:1:-1];

    theAnimation.fromValue = @(theView.depth);
    theAnimation.toValue = @(theDepth);
    theAnimation.duration = 5.0;
    theAnimation.timingFunction = theFunction;
    theView.depth = theDepth;
    [theView.layer addAnimation:theAnimation forKey:@"depth"];
#else
    theView.depth = theDepth;
#endif

    //
    // ======== ENDE DEMO BEREICH ========
    //

    [self updateDepthLabel:inSender];
}

- (IBAction)updateSize:(UISlider *)inSender {
    CALayer *theLayer = self.pointLayer;
    CGFloat theScale = inSender.value;

    theLayer.transform = CATransform3DMakeScale(theScale, theScale, theScale);
    [self updateSizeLabel:inSender];
}

- (IBAction)updateSizeLabel:(UISlider *)inSender {
    self.sizeLabel.text = [NSString stringWithFormat:@"%.1f%%", inSender.value * 100.0];
}

@end
