//
//  KochLayer.m
//  ScrollView
//
//  Created by Clemens Wagner on 05.02.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "KochLayer.h"

static NSString * const kDepthKey = @"depth";

@implementation KochLayer

@dynamic depth;

+ (id)defaultValueForKey:(NSString *)inKey {
    if([inKey isEqualToString:kDepthKey]) {
        return [NSNumber numberWithFloat:0.0];
    }
    else {
        return [super defaultValueForKey:inKey];
    }
}

+ (BOOL)needsDisplayForKey:(NSString *)inKey {
    return [inKey isEqualToString:kDepthKey] || [super needsDisplayForKey:inKey];
}

- (void)drawInContext:(CGContextRef)inContext {
    CGRect theBounds = self.bounds;
    CGFloat theRadius = fminf(CGRectGetWidth(theBounds), CGRectGetHeight(theBounds)) / 2.0;
    CGPoint thePoints[3] = {
        [self pointWithRadius:theRadius angle:0.0],
        [self pointWithRadius:theRadius angle:2 * M_PI / 3.0],
        [self pointWithRadius:theRadius angle:4 * M_PI / 3.0]
    };
    CGMutablePathRef thePath = CGPathCreateMutable();

    CGContextSetLineWidth(inContext, 0.5);
    CGContextSetLineCap(inContext, kCGLineCapRound);
    CGContextSetLineJoin(inContext, kCGLineJoinRound);
    CGContextSetFillColorWithColor(inContext, [UIColor blackColor].CGColor);
    CGPathMoveToPoint(thePath, NULL, thePoints[0].x, thePoints[0].y);
    for(int i = 0; i < 3; ++i) {
        [self addPointsToPath:thePath fromPoint:thePoints[i] toPoint:thePoints[(i + 1) % 3] withDepth:self.depth];
    }
    CGContextAddPath(inContext, thePath);
    CGContextFillPath(inContext);
    CGPathRelease(thePath);
}

- (void)curveWithWeight:(float)inWeight points:(CGPoint *)inoutPoints  {
    CGPoint theFromPoint = inoutPoints[0];
    CGPoint theToPoint = inoutPoints[4];
    CGFloat theFactor = inWeight / (2 * sqrt(3));
    CGSize theDelta = CGSizeMake(theToPoint.x - theFromPoint.x, theToPoint.y - theFromPoint.y);

    inoutPoints[1] = CGPointMake(theFromPoint.x + theDelta.width / 3,
                                 theFromPoint.y + theDelta.height / 3);
    inoutPoints[2] = CGPointMake(theFromPoint.x + theDelta.width / 2 + theFactor * theDelta.height,
                                 theFromPoint.y + theDelta.height / 2 - theFactor * theDelta.width);
    inoutPoints[3] = CGPointMake(theToPoint.x - theDelta.width / 3,
                                 theToPoint.y - theDelta.height / 3);
}

- (CGPoint)midPoint {
    CGRect theBounds = self.bounds;

    return CGPointMake(CGRectGetMidX(theBounds), CGRectGetMidY(theBounds));
}

- (CGPoint)pointWithRadius:(CGFloat)inRadius angle:(CGFloat)inAngle {
    CGPoint theCenter = [self midPoint];

    return CGPointMake(theCenter.x + inRadius * sin(inAngle),
                       theCenter.y - inRadius * cos(inAngle));
}

- (void)addPointsToPath:(CGMutablePathRef)inoutPath
              fromPoint:(CGPoint)inFromPoint
                toPoint:(CGPoint)inToPoint
              withDepth:(float)inDepth {
    CGPoint thePoints[5];

    thePoints[0] = inFromPoint;
    thePoints[4] = inToPoint;
    if(inDepth <= 1.0) {
        [self curveWithWeight:inDepth points:thePoints];
        for(NSUInteger i = 1; i < 5; ++i) {
            CGPathAddLineToPoint(inoutPath, NULL, thePoints[i].x, thePoints[i].y);
        }
    }
    else {
        CGFloat theDepth = inDepth - 1;

        [self curveWithWeight:1.0 points:thePoints];
        for(NSUInteger i = 0; i < 4; ++i) {
            [self addPointsToPath:inoutPath fromPoint:thePoints[i] toPoint:thePoints[i + 1] withDepth:theDepth];
        }
    }
}

@end
